@@ -12,16 +12,20 @@ from can.typechecking import AutoDetectedConfig, CanFilters
 
 from can_cando.CANdoImport import (
     CAN_MSG_TIMESTAMP_RESOLUTION,
+    CANDO_BUS_LOAD_STATUS,
     CANDO_CAN_BUFFER_LENGTH,
+    CANDO_DATE_STATUS,
     CANDO_DEVICE_STATUS,
     CANDO_ID_11_BIT,
     CANDO_ID_29_BIT,
+    CANDO_LATEST_SW,
     CANDO_MAX_FILTERS,
     CANDO_NO_STATUS,
     CANDO_PID,
     # CANDO_REPEAT_TIME_MAP,
     CANDO_RUN,
     CANDO_STOP,
+    CANDOISO_LATEST_SW,
     CANDOISO_PID,
     CANdoBusStsErr,
     CANdoCANBufferPtrType,
@@ -36,12 +40,12 @@ from can_cando.CANdoImport import (
     CANdoInitializationError,
     CANdoMode,
     CANdoOpenDevice,
-    # CANdoRequestBusLoadStatus,
-    # CANdoRequestDateStatus,
     CANdoOperationError,
     CANdoPIDType,
     CANdoReceive,
     CANdoRepeatTime,
+    CANdoRequestBusLoadStatus,
+    CANdoRequestDateStatus,
     CANdoRequestStatus,
     CANdoSetBaudRate,
     CANdoSetFilters,
@@ -153,7 +157,6 @@ class CANdoBus(BusABC):
         channel: Optional[int] = None,
         can_filters: Optional[CanFilters] = None,
         bitrate: Optional[int] = default_bitrate,
-        is_candoiso: Optional[bool] = False,
         **kwargs: object,
     ) -> None:
         self.channel = channel
@@ -174,6 +177,7 @@ class CANdoBus(BusABC):
             f"Driver = v{DriverVersion.value / 10}\n"
         )
 
+        self.devices_PIDs: List[bytes] = []
         self._detect_cando_iso()
 
         if channel is None:
@@ -191,7 +195,7 @@ class CANdoBus(BusABC):
                 f"Invalid channel number: {channel}, only {self.NoOfDevices.value} devices detected and values start at 0",
             )
 
-        self.is_candoiso = is_candoiso
+        self.is_candoiso = self.devices_PIDs[channel] == CANDOISO_PID
         self.bit_timing = cando_get_timing(bitrate or self.default_bitrate)
 
         # CANdoOpen function
@@ -494,18 +498,36 @@ class CANdoBus(BusABC):
         detected on the CAN bus, or if there is an internal system error within the
         CANdo device.)
         """
+        # TODO: Use "CANdoRequestStatus" function
 
-    def cando_get_date_status(self) -> None:
+    def cando_get_date_status(self) -> Optional[Tuple[int, int, int]]:
         """Request the date of manufacture of the CANdo device.
 
         After sending this command to CANdo, the date status is transmitted back to
         the PC in <1ms. To read the status, call the CANdoReceive(...) function &
         interrogate the NewFlag & status information within the TCANdoStatus
         parameter.
-        """
-        # TODO: Use "CANdoRequestDateStatus" function
 
-    def cando_request_bus_load_status(self) -> None:
+        :return: Tuple containing the manufacturing date as (day, month, year).
+        """
+        ret = None
+        with self._recv_lock:  # Ensure that the receiving thread does not interfere
+            if CANdoRequestDateStatus(self.CANdoUSBPtr) == CANdoFuncRetCode.CANDO_SUCCESS:
+                sleep(0.001)
+                self.CANdoStatus.NewFlag = CANDO_NO_STATUS  # Clear status flag
+                if (
+                    CANdoReceive(
+                        self.CANdoUSBPtr,
+                        self.CANdoCANBufferPtr,
+                        self.CANdoStatusPtr,
+                    )
+                    == CANdoFuncRetCode.CANDO_SUCCESS
+                ) and self.CANdoStatus.NewFlag == CANDO_DATE_STATUS:  # type: ignore
+                    ret = (self.CANdoStatus.SoftwareVersion, self.CANdoStatus.Status, self.CANdoStatus.BusState)
+
+        return ret
+
+    def cando_request_bus_load_status(self) -> Optional[Tuple[float, int, int]]:
         """Request the CAN bus loading as calculated by connected device.
 
         The bus load is calculated every second, while the device is running.
@@ -514,8 +536,28 @@ class CANdoBus(BusABC):
         back to the PC in <1ms. To read the status, call the CANdoReceive(...)
         function & interrogate the NewFlag & status information within the
         TCANdoStatus parameter.
+
+        :return: Tuple containing bus load percentage, CAN RX error count, CAN TX error count.
         """
-        # TODO: Use the "CANdoRequestBusLoadStatus" function
+        ret = None
+        with self._recv_lock:  # Ensure that the receiving thread does not interfere
+            if CANdoRequestBusLoadStatus(self.CANdoUSBPtr) == CANdoFuncRetCode.CANDO_SUCCESS:
+                sleep(0.001)
+                self.CANdoStatus.NewFlag = CANDO_NO_STATUS  # Clear status flag
+                if (
+                    CANdoReceive(
+                        self.CANdoUSBPtr,
+                        self.CANdoCANBufferPtr,
+                        self.CANdoStatusPtr,
+                    )
+                    == CANdoFuncRetCode.CANDO_SUCCESS
+                ) and self.CANdoStatus.NewFlag == CANDO_BUS_LOAD_STATUS:  # type: ignore
+                    ret = (
+                        self.CANdoStatus.HardwareVersion + self.CANdoStatus.SoftwareVersion / 10,
+                        self.CANdoStatus.Status,
+                        self.CANdoStatus.BusState,
+                    )
+        return ret
 
     def get_status_description(self) -> str:
         """Request the CAN bus & internal CANdo status.
@@ -558,8 +600,25 @@ class CANdoBus(BusABC):
                     sts += f"Status Err: {CANdoStsErr(self.CANdoStatus.Status)}\n"
                     sts += f"Bus State Err: {CANdoBusStsErr(self.CANdoStatus.BusState)}\n"
 
+                    self.cando_check_software(self.CANdoStatus.HardwareVersion, self.CANdoStatus.SoftwareVersion)
+
         return sts
 
+    def cando_check_software(self, hw_ver: int, sw_ver: int) -> None:
+        device = "CANdoISO" if self.is_candoiso else "CANdo"
+        try:
+            msg = ""
+            if self.is_candoiso:
+                if CANDOISO_LATEST_SW[hw_ver] > sw_ver:
+                    msg = f"Newer software version available for {device}: v{CANDOISO_LATEST_SW[hw_ver] / 10}"
+            else:  # noqa: PLR5501
+                if CANDO_LATEST_SW[hw_ver] > sw_ver:
+                    msg = f"Newer software version available for {device}: v{CANDO_LATEST_SW[hw_ver] / 10}"
+            if msg != "":
+                log.debug(msg)
+        except KeyError:
+            log.error(f"Unknown hardware version {hw_ver} for device {device}")  # noqa: TRY400
+
     def print_status(self) -> None:
         log.info(self.get_status_description())
 
@@ -612,6 +671,7 @@ class CANdoBus(BusABC):
                     log.debug(f"  Device no. {CANdoNo} >\n    PID = 0x{self.CANdoPID.value.decode('utf-8').upper()}")
                     if self.CANdoPID.value not in [CANDO_PID, CANDOISO_PID]:
                         log.warning(f"    Device no. {CANdoNo} is not a CANdo(ISO) device!")
+                    self.devices_PIDs.append(self.CANdoPID.value)
 
                     # H/W type & S/N
                     log.debug(
